@@ -725,7 +725,9 @@ plot.kroc <- function(x, add=FALSE, add.roc.ref=FALSE, ylab="True positive rate
   }
 }
 
+#############################################################################
 ## summary method
+#############################################################################
 summary.kroc <- function(object, ...)
 {
   cat("Summary measures for ROC curve\nAUC =", signif(object$indices$auc, ...), "\n")
@@ -733,3 +735,17 @@ summary.kroc <- function(object, ...)
   cat(paste("(LR-, LR+) = (",  signif(object$indices$LR$minus, ...), ", ", signif(object$indices$LR$plus, ...),")\n\n",sep=""))
 }
 
+
+#############################################################################
+## predict methods
+#############################################################################
+predict.kcde <- function(object, ..., x)
+{
+  return(predict.kde(object, ...,x=x))
+}
+
+predict.kroc <- function(object, ..., x)
+{
+  return(predict.kde(object, ..., x=x))
+}
+
